"""
-*- coding: utf-8 -*-
@Author  : Ora
@Time    : 2020/12/30 9:40 下午
@Software: PyCharm
@File    : test_experimental.py
@E-mail  : victor.xsyang@gmail.com
"""
from typing import Any

import pytest

from opts import _experimental
from opts.errors import ExperimentalWarning


def _sample_func() -> int:

    return 10


class _Sample(object):
    def __init__(self, a: Any, b: Any, c: Any) -> None:
        pass

    def _method(self) -> None:
        """summary

        detail
        """
        pass

    def _method_expected(self) -> None:
        """summary

        detail

        .. note::
            Added in v1.1.0 as an experimental feature. The interface may change in newer versions
            without prior notice.
        """
        pass


@pytest.mark.parametrize("version", ["1.1", 100, None])
def test_experimental_raises_error_for_invalid_version(version: Any) -> None:
    with pytest.raises(ValueError):
        _experimental.experimental(version)


def test_experimental_decorator() -> None:
    version = "1.1.0"
    decorator_experimental = _experimental.experimental(version)
    assert callable(decorator_experimental)

    decorated_func = decorator_experimental(_sample_func)
    assert decorated_func.__name__ == _sample_func.__name__
    assert decorated_func.__doc__ == _experimental._EXPERIMENTAL_NOTE_TEMPLATE.format(ver=version)

    with pytest.warns(ExperimentalWarning):
        decorated_func()


def test_experimental_method_decorator() -> None:
    version = "1.1.0"
    decorator_experimental = _experimental.experimental(version)
    assert callable(decorator_experimental)

    decorated_method = decorator_experimental(_Sample._method)

    print(decorated_method.__doc__)
    print(_Sample._method_expected.__doc__)
    assert decorated_method.__name__ == _Sample._method.__name__
    assert decorated_method.__doc__ == _Sample._method_expected.__doc__

    with pytest.warns(ExperimentalWarning):
        decorated_method(None)


def test_experimental_class_decorator() -> None:
    version = "1.1.0"
    decorator_experimental = _experimental.experimental(version)
    assert callable(decorator_experimental)

    decorated_class = decorator_experimental(_Sample)
    assert decorated_class.__name__ == "_Sample"
    assert decorated_class.__init__.__name__ == "__init__"
    assert decorated_class.__doc__ == _experimental._EXPERIMENTAL_NOTE_TEMPLATE.format(ver=version)

    with pytest.warns(ExperimentalWarning):
        decorated_class("a", "b", "c")


def test_experimental_class_decorator_name() -> None:

    name = "foo"
    decorator_experimental = _experimental.experimental("1.1.0", name=name)
    decorated_sample = decorator_experimental(_Sample)

    with pytest.warns(ExperimentalWarning) as record:
        decorated_sample("a", "b", "c")

    assert name in record.list[0].message.args[0]


def test_experimental_decorator_name() -> None:

    name = "bar"
    decorator_experimental = _experimental.experimental("1.1.0", name=name)
    decorated_sample_func = decorator_experimental(_sample_func)

    with pytest.warns(ExperimentalWarning) as record:
        decorated_sample_func()

    assert name in record.list[0].message.args[0]
